unit rkVistaPanel;

//  VistaPanel by Roy Magne Klever
//   2010 by Roy Magne Klever. All rights reserved
//
//  This file is not distributable without permission by Roy Magne Klever
//  WEB: www.rmklever.com
//  Mail: roymagne@rmklever.com
//
//  version 1.1, April 2010
//
//  MPL 1.1 licenced

interface

uses
  Windows, SysUtils, Classes, Controls, ExtCtrls, Graphics, Forms, Messages;

type
  TrkVistaPanel = class(TCustomControl)
  private
    FBorderStyle: TBorderStyle;
    FBottomColor: TColor;
    FCenter: Boolean;
    FColor1: TColor;
    FColor2: TColor;
    FColor3: TColor;
    FColor4: TColor;
    FColorBorder: TColor;
    FBottomLine: Boolean;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMIsToolControl(var Message: TMessage); message CM_ISTOOLCONTROL;
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetColor1(const Value: TColor);
    procedure SetColor2(const Value: TColor);
    procedure SetColor3(const Value: TColor);
    procedure SetColor4(const Value: TColor);
    procedure WMERASEBKGND(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMWindowPosChanged(var Message: TWMWindowPosChanged); message
      WM_WINDOWPOSCHANGED;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure SetColorFrame(const Value: TColor);
    procedure SetCenter(const Value: Boolean);
    procedure SetBottomLine(const Value: Boolean);
    procedure SetBottomColor(const Value: TColor);
    { Private declarations }
  protected
    { Protected declarations }
    procedure CreateParams(var Params: TCreateParams); override;
    procedure PaintWindow(DC: HDC); override;
    procedure Resize; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GlassGradient(ACanvas: TCanvas; AWidth, AHeight: Integer; c1, c2,
      c3, c4: TColor; Center: Boolean);
  published
    { Published declarations }
    property Align;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelKind;
    property BevelWidth;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle
      default bsNone;
    property BorderWidth;
    property BottomLine: Boolean read FBottomLine write SetBottomLine default True;
    property Center: Boolean read FCenter write SetCenter default False;
    property BottomColor: TColor read FBottomColor write SetBottomColor default clBtnFace;
    property Color1: TColor read FColor1 write SetColor1 default clBlack;
    property Color2: TColor read FColor2 write SetColor2 default clBlack;
    property Color3: TColor read FColor3 write SetColor3 default clBlack;
    property Color4: TColor read FColor4 write SetColor4 default clBlack;
    property ColorFrame: TColor read FColorBorder write SetColorFrame default
      $00C8BA90;
    property Font;
    property Left;
    property Top;
    property Width;
    property Height;
    property ParentBackground;
    property ShowHint;
    property OnEnter;
    property OnExit;
    property OnResize;
    property OnClick;
    property OnDblClick;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnKeyDown;
    property OnKeyUp;
  end;

procedure Register;

implementation

procedure TrkVistaPanel.WMERASEBKGND(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure TrkVistaPanel.WMPaint(var Message: TWMPaint);
begin
  PaintHandler(Message);
end;

procedure TrkVistaPanel.CMCtl3DChanged(var Message: TMessage);
begin
  if NewStyleControls and (FBorderStyle = bsSingle) then
    RecreateWnd;
  inherited;
end;

procedure TrkVistaPanel.CMIsToolControl(var Message: TMessage);
begin
  Message.Result := 1;
end;

procedure TrkVistaPanel.WMWindowPosChanged(var Message: TWMWindowPosChanged);
begin
  Invalidate;
  inherited;
  if not (csLoading in ComponentState) then
    Resize;
end;

procedure TrkVistaPanel.SetBorderStyle(Value: TBorderStyle);
begin
  if Value <> FBorderStyle then
  begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

procedure TrkVistaPanel.SetBottomColor(const Value: TColor);
begin
  FBottomColor := Value;
  Invalidate;
end;

procedure TrkVistaPanel.SetBottomLine(const Value: Boolean);
begin
  FBottomLine := Value;
  Invalidate;
end;

procedure TrkVistaPanel.SetCenter(const Value: Boolean);
begin
  FCenter := Value;
  Invalidate;
end;

procedure TrkVistaPanel.SetColor1(const Value: TColor);
begin
  FColor1 := Value;
  Invalidate;
end;

procedure TrkVistaPanel.SetColor2(const Value: TColor);
begin
  FColor2 := Value;
  Invalidate;
end;

procedure TrkVistaPanel.SetColor3(const Value: TColor);
begin
  FColor3 := Value;
  Invalidate;
end;

procedure TrkVistaPanel.SetColor4(const Value: TColor);
begin
  FColor4 := Value;
  Invalidate;
end;

procedure TrkVistaPanel.SetColorFrame(const Value: TColor);
begin
  FColorBorder := Value;
  Invalidate;
end;

procedure TrkVistaPanel.GlassGradient(ACanvas: TCanvas; AWidth, AHeight:
  Integer; c1, c2, c3, c4: TColor; Center: Boolean);
type
  PRGB = ^TRGB;
  TRGB = record b, g, r: Byte;
  end;
  PRGBArray = ^TRGBArray;
  TRGBARRAY = array[0..0] of TRGB;
var
  rc1, gc1, bc1, rc2, gc2, bc2, rc3, gc3, bc3: Integer;
  x, y, w, h: Integer;
  i, w1: Integer;
  Row: PRGBArray;
  C: TRGB;
  bmp: TBitmap;
  slMain, slSize, slPtr: Integer;
  Color: Integer;
  QCol: array of TRGB;
  r, g, b: Byte;
begin
  bmp := TBitmap.Create;
  bmp.PixelFormat := pf24Bit;
  bmp.Width := AWidth;
  bmp.Height := AHeight;
  h := bmp.Height;
  w := bmp.Width;
  SetLength(QCol, h);
  // Get colors for first gradient
  Color := ColorToRGB(c1);
  rc1 := Byte(Color);
  gc1 := Byte(Color shr 8);
  bc1 := Byte(Color shr 16);
  Color := ColorToRGB(c2);
  rc2 := Byte(Color);
  gc2 := Byte(Color shr 8);
  bc2 := Byte(Color shr 16);
  rc3 := rc1 + (((rc2 - rc1) * 7) div 2);
  gc3 := gc1 + (((gc2 - gc1) * 7) div 2);
  bc3 := bc1 + (((bc2 - bc1) * 7) div 2);
  if rc3 < 0 then
    rc3 := 0
  else if rc3 > 255 then
    rc3 := 255;
  if gc3 < 0 then
    gc3 := 0
  else if gc3 > 255 then
    gc3 := 255;
  if bc3 < 0 then
    bc3 := 0
  else if bc3 > 255 then
    bc3 := 255;
  // Calc first gradient
  y := h div 2;
  for i := 0 to y - 1 do
  begin
    C.r := Byte(rc1 + (((rc2 - rc1) * (i)) div y));
    C.g := Byte(gc1 + (((gc2 - gc1) * (i)) div y));
    C.b := Byte(bc1 + (((bc2 - bc1) * (i)) div y));
    QCol[i] := C;
  end;
  if rc2 > rc1 then
  begin
    rc3 := rc2;
    gc3 := gc2;
    bc3 := bc2;
  end;
  for i := y to h - 1 do
  begin
    C.r := Byte(rc3 + (((rc2 - rc3) * (i)) div h));
    C.g := Byte(gc3 + (((gc2 - gc3) * (i)) div h));
    C.b := Byte(bc3 + (((bc2 - bc3) * (i)) div h));
    QCol[i] := C;
  end;
  // First gradient done
  // Get colors for second gradient
  Color := ColorToRGB(c3);
  rc1 := Byte(Color);
  gc1 := Byte(Color shr 8);
  bc1 := Byte(Color shr 16);
  Color := ColorToRGB(c4);
  rc2 := Byte(Color);
  gc2 := Byte(Color shr 8);
  bc2 := Byte(Color shr 16);
  w1 := w - 1;
  if Center then
    w := (w shr 1) + (w and 1);
  // Init scanline accsess
  slMain := Integer(bmp.ScanLine[0]);
  slSize := Integer(bmp.ScanLine[1]) - slMain;
  // Paint gradient
  for x := 0 to w - 1 do
  begin
    C.b := Byte(bc1 + (((bc2 - bc1) * x) div w));
    C.g := Byte(gc1 + (((gc2 - gc1) * x) div w));
    C.r := Byte(rc1 + (((rc2 - rc1) * x) div w));
    slPtr := slMain;
    for y := 0 to h - 1 do
    begin
      Row := PRGBArray(slPtr);
      r := QCol[y].r;
      g := QCol[y].g;
      b := QCol[y].b;
      Row[x].r := (C.r - r) shr 1 + r;
      Row[x].g := (C.g - g) shr 1 + g;
      Row[x].b := (C.b - b) shr 1 + b;
      if (Center) and (x < (w1 - x)) then
      begin
        Row[w1 - x].r := (C.r - r) shr 1 + r;
        Row[w1 - x].g := (C.g - g) shr 1 + g;
        Row[w1 - x].b := (C.b - b) shr 1 + b;
      end;
      slPtr := slPtr + slSize;
    end;
  end;
  QCol := nil;
  if FBottomLine then
  begin
    bmp.Canvas.Pen.Color:= FBottomColor;
    bmp.Canvas.MoveTo(0, AHeight - 2);
    bmp.Canvas.LineTo(AWidth, AHeight - 2);
  end;
  Canvas.Draw(0, 0, bmp);
  bmp.Free;
end;

procedure TrkVistaPanel.PaintWindow(DC: HDC);
begin
  Canvas.Lock;
  try
    Canvas.Handle := DC;
    try
      GlassGradient(Canvas, CLientWidth, ClientHeight, FColor1, FColor2,
        FColor3, FColor4, FCenter);
      Canvas.Pen.Color := FColorBorder;
      Canvas.Brush.Style := bsClear;
      Canvas.Rectangle(ClientRect);
    finally
      Canvas.Handle := 0;
    end;
  finally
    Canvas.Unlock;
  end;
end;

constructor TrkVistaPanel.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csOpaque, csAcceptsControls, csReplicatable,
    csNeedsBorderPaint];
  FBorderStyle := bsNone;
  FBottomLine:= True;
  FBottomColor:= clBtnFace;
  FColor1 := clWhite;
  FColor2 := clSilver;
  FColor3 := clYellow;
  FColor4 := clTeal;
  FColorBorder := clGreen;
  FCenter := False;
end;

destructor TrkVistaPanel.Destroy;
begin
  inherited Destroy;
end;

procedure TrkVistaPanel.Resize;
begin
  Invalidate;
end;

procedure TrkVistaPanel.CreateParams(var Params: TCreateParams);
const
  BorderStyles: array[TBorderStyle] of DWord = (0, WS_BORDER);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style or BorderStyles[FBorderStyle];
    if NewStyleControls and Ctl3D and (FBorderStyle = bsSingle) then
    begin
      Style := Style and not WS_BORDER;
      ExStyle := ExStyle or WS_EX_CLIENTEDGE;
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TrkVistaPanel]);
end;

end.

